"""Tests for PGFPlots generation."""

import pytest
from texer import PGFPlot, Axis, AddPlot, Coordinates, Ref, Iter, evaluate, GroupPlot, NextGroupPlot


class TestCoordinates:
    def test_static_coordinates(self):
        coords = Coordinates([(0, 1), (1, 2), (2, 4)])
        result = coords.render({})
        assert result == "coordinates {(0, 1) (1, 2) (2, 4)}"

    def test_dynamic_coordinates(self):
        coords = Coordinates(
            Iter(Ref("points"), x=Ref("x"), y=Ref("y"))
        )
        data = {"points": [{"x": 0, "y": 1}, {"x": 1, "y": 4}]}
        result = coords.render(data)
        assert result == "coordinates {(0, 1) (1, 4)}"

    def test_3d_coordinates(self):
        coords = Coordinates([(0, 0, 1), (1, 1, 2)])
        result = coords.render({})
        assert result == "coordinates {(0, 0, 1) (1, 1, 2)}"


class TestAddPlot:
    def test_simple_addplot(self):
        plot = AddPlot(
            color="blue",
            mark="*",
            coords=Coordinates([(0, 1), (1, 2)]),
        )
        result = plot.render({})
        assert "\\addplot" in result
        assert "color=blue" in result
        assert "mark=*" in result
        assert "coordinates" in result

    def test_expression_plot(self):
        plot = AddPlot(
            expression="x^2",
            domain="0:5",
            samples=100,
        )
        result = plot.render({})
        assert "domain=0:5" in result
        assert "samples=100" in result
        assert "{x^2}" in result

    def test_only_marks(self):
        plot = AddPlot(
            only_marks=True,
            coords=Coordinates([(0, 1)]),
        )
        result = plot.render({})
        assert "only marks" in result


class TestAxis:
    def test_basic_axis(self):
        axis = Axis(
            xlabel="Time (s)",
            ylabel="Value",
            plots=[AddPlot(coords=Coordinates([(0, 1)]))],
        )
        result = axis.render({})
        assert "\\begin{axis}" in result
        assert "xlabel={Time (s)}" in result
        assert "ylabel=Value" in result  # No braces for simple value
        assert "\\end{axis}" in result

    def test_axis_with_limits(self):
        axis = Axis(
            xmin=0,
            xmax=10,
            ymin=-5,
            ymax=5,
            plots=[],
        )
        result = axis.render({})
        assert "xmin=0" in result
        assert "xmax=10" in result
        assert "ymin=-5" in result
        assert "ymax=5" in result

    def test_axis_with_legend(self):
        axis = Axis(
            plots=[AddPlot(coords=Coordinates([(0, 1)]))],
            legend=["Series A"],
            legend_pos="north west",
        )
        result = axis.render({})
        assert "legend pos={north west}" in result
        assert "\\legend{Series A}" in result

    def test_axis_with_grid(self):
        axis = Axis(grid=True, plots=[])
        result = axis.render({})
        assert "grid=major" in result

    def test_dynamic_labels(self):
        axis = Axis(
            xlabel=Ref("x_label"),
            ylabel=Ref("y_label"),
            plots=[],
        )
        data = {"x_label": "Distance (m)", "y_label": "Speed (m/s)"}
        result = axis.render(data)
        assert "xlabel={Distance (m)}" in result
        assert "ylabel={Speed (m/s)}" in result


class TestPGFPlot:
    def test_complete_plot(self):
        plot = PGFPlot(
            Axis(
                xlabel="X",
                ylabel="Y",
                plots=[
                    AddPlot(
                        color="blue",
                        mark="*",
                        coords=Coordinates([(0, 0), (1, 1), (2, 4)]),
                    )
                ],
            )
        )
        result = plot.render({})
        assert "\\begin{tikzpicture}" in result
        assert "\\begin{axis}" in result
        assert "\\addplot" in result
        assert "\\end{axis}" in result
        assert "\\end{tikzpicture}" in result

    def test_plot_with_scale(self):
        plot = PGFPlot(
            Axis(plots=[]),
            scale=0.8,
        )
        result = plot.render({})
        assert "scale=0.8" in result

    def test_with_preamble(self):
        plot = PGFPlot(Axis(plots=[]))
        result = plot.with_preamble()
        assert "\\documentclass{standalone}" in result
        assert "\\usepackage{pgfplots}" in result
        assert "\\pgfplotsset{compat=1.18}" in result
        assert "\\begin{document}" in result
        assert "\\end{document}" in result


class TestDynamicPlot:
    def test_plot_from_data(self):
        """Test the key use case: generating plot from data."""
        plot = PGFPlot(
            Axis(
                xlabel="Time (s)",
                ylabel="Temperature (K)",
                legend_pos="north west",
                plots=[
                    AddPlot(
                        color="blue",
                        mark="*",
                        coords=Coordinates(
                            Iter(Ref("data_points"), x=Ref("t"), y=Ref("temp"))
                        ),
                    )
                ],
                legend=[Ref("legend_label")],
            )
        )

        data = {
            "data_points": [
                {"t": 0, "temp": 300},
                {"t": 1, "temp": 310},
                {"t": 2, "temp": 315},
            ],
            "legend_label": "Experiment 1",
        }

        result = evaluate(plot, data)

        assert "xlabel={Time (s)}" in result
        assert "ylabel={Temperature (K)}" in result
        assert "coordinates {(0, 300) (1, 310) (2, 315)}" in result
        assert "\\legend{Experiment 1}" in result


class TestGroupPlot:
    def test_basic_groupplot(self):
        """Test basic groupplot creation with 2x2 layout."""
        groupplot = GroupPlot(
            group_size="2 by 2",
            plots=[
                NextGroupPlot(
                    title="Plot 1",
                    plots=[AddPlot(coords=Coordinates([(0, 1), (1, 2)]))],
                ),
                NextGroupPlot(
                    title="Plot 2",
                    plots=[AddPlot(coords=Coordinates([(0, 2), (1, 3)]))],
                ),
                NextGroupPlot(
                    title="Plot 3",
                    plots=[AddPlot(coords=Coordinates([(0, 3), (1, 4)]))],
                ),
                NextGroupPlot(
                    title="Plot 4",
                    plots=[AddPlot(coords=Coordinates([(0, 4), (1, 5)]))],
                ),
            ],
        )
        result = groupplot.render({})
        assert "\\begin{groupplot}" in result
        assert "group size={2 by 2}" in result
        assert "\\nextgroupplot" in result
        assert result.count("\\nextgroupplot") == 4
        assert "\\end{groupplot}" in result

    def test_groupplot_with_common_options(self):
        """Test groupplot with common options applied to all subplots."""
        groupplot = GroupPlot(
            group_size="1 by 2",
            width="6cm",
            height="4cm",
            xmin=0,
            xmax=10,
            plots=[
                NextGroupPlot(plots=[]),
                NextGroupPlot(plots=[]),
            ],
        )
        result = groupplot.render({})
        assert "width=6cm" in result
        assert "height=4cm" in result
        assert "xmin=0" in result
        assert "xmax=10" in result

    def test_groupplot_with_separations(self):
        """Test groupplot with custom spacing."""
        groupplot = GroupPlot(
            group_size="2 by 1",
            horizontal_sep="2cm",
            vertical_sep="1.5cm",
            plots=[
                NextGroupPlot(plots=[]),
                NextGroupPlot(plots=[]),
            ],
        )
        result = groupplot.render({})
        assert "horizontal sep=2cm" in result
        assert "vertical sep=1.5cm" in result

    def test_groupplot_with_label_positions(self):
        """Test groupplot with custom label positions."""
        groupplot = GroupPlot(
            group_size="2 by 2",
            xlabels_at="edge bottom",
            ylabels_at="edge left",
            plots=[
                NextGroupPlot(xlabel="X", ylabel="Y", plots=[]),
                NextGroupPlot(plots=[]),
                NextGroupPlot(plots=[]),
                NextGroupPlot(plots=[]),
            ],
        )
        result = groupplot.render({})
        assert "xlabels at={edge bottom}" in result
        assert "ylabels at={edge left}" in result

    def test_nextgroupplot_with_options(self):
        """Test nextgroupplot with various options."""
        plot = NextGroupPlot(
            title="Test Plot",
            xlabel="X axis",
            ylabel="Y axis",
            xmin=0,
            xmax=10,
            ymin=0,
            ymax=5,
            grid=True,
            legend=["Series A"],
            plots=[AddPlot(coords=Coordinates([(0, 1), (5, 3), (10, 5)]))],
        )
        result = plot.render({})
        assert "\\nextgroupplot" in result
        assert "title={Test Plot}" in result
        assert "xlabel={X axis}" in result
        assert "ylabel={Y axis}" in result
        assert "xmin=0" in result
        assert "xmax=10" in result
        assert "ymin=0" in result
        assert "ymax=5" in result
        assert "grid=major" in result
        assert "\\legend{Series A}" in result

    def test_pgfplot_with_groupplot(self):
        """Test complete PGFPlot with groupplot."""
        plot = PGFPlot(
            GroupPlot(
                group_size="1 by 2",
                plots=[
                    NextGroupPlot(
                        title="Left",
                        plots=[AddPlot(coords=Coordinates([(0, 0), (1, 1)]))],
                    ),
                    NextGroupPlot(
                        title="Right",
                        plots=[AddPlot(coords=Coordinates([(0, 1), (1, 0)]))],
                    ),
                ],
            )
        )
        result = plot.render({})
        assert "\\begin{tikzpicture}" in result
        assert "\\begin{groupplot}" in result
        assert "\\nextgroupplot" in result
        assert "\\end{groupplot}" in result
        assert "\\end{tikzpicture}" in result

    def test_groupplot_dynamic_data(self):
        """Test groupplot with dynamic data from Ref and Iter."""
        groupplot = GroupPlot(
            group_size="1 by 2",
            plots=[
                NextGroupPlot(
                    title=Ref("title1"),
                    plots=[
                        AddPlot(
                            coords=Coordinates(
                                Iter(Ref("data1"), x=Ref("x"), y=Ref("y"))
                            )
                        )
                    ],
                ),
                NextGroupPlot(
                    title=Ref("title2"),
                    plots=[
                        AddPlot(
                            coords=Coordinates(
                                Iter(Ref("data2"), x=Ref("x"), y=Ref("y"))
                            )
                        )
                    ],
                ),
            ],
        )

        data = {
            "title1": "First Plot",
            "title2": "Second Plot",
            "data1": [{"x": 0, "y": 0}, {"x": 1, "y": 1}],
            "data2": [{"x": 0, "y": 1}, {"x": 1, "y": 0}],
        }

        result = groupplot.render(data)
        assert "title={First Plot}" in result
        assert "title={Second Plot}" in result
        assert "coordinates {(0, 0) (1, 1)}" in result
        assert "coordinates {(0, 1) (1, 0)}" in result

    def test_groupplot_with_preamble(self):
        """Test that groupplot includes the groupplots library in preamble."""
        plot = PGFPlot(
            GroupPlot(
                group_size="1 by 1",
                plots=[NextGroupPlot(plots=[])],
            )
        )
        result = plot.with_preamble()
        assert "\\usepgfplotslibrary{groupplots}" in result
        assert "\\documentclass{standalone}" in result
        assert "\\usepackage{pgfplots}" in result



"""PGFPlots classes for LaTeX figure generation."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any

from texer.specs import Spec, Iter, resolve_value
from texer.utils import format_options, indent


@dataclass
class Coordinates:
    """Coordinates for a plot.

    Examples:
        # Static coordinates
        Coordinates([(0, 1), (1, 2), (2, 4)])

        # Dynamic coordinates from data
        Coordinates(Iter(Ref("points"), x=Ref("x"), y=Ref("y")))

        # 3D coordinates
        Coordinates([(0, 0, 1), (1, 1, 2)])
    """

    source: list[tuple[Any, ...]] | Iter | Spec

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render coordinates to LaTeX."""
        if scope is None:
            scope = {}

        # Resolve the source
        if isinstance(self.source, (Iter, Spec)):
            points = self.source.resolve(data, scope)
        else:
            points = self.source

        # Format coordinates
        coord_strs = []
        for point in points:
            if isinstance(point, tuple):
                coord_strs.append(f"({', '.join(str(v) for v in point)})")
            else:
                # Single value (rare case)
                coord_strs.append(f"({point})")

        return "coordinates {" + " ".join(coord_strs) + "}"


@dataclass
class AddPlot:
    """An \\addplot command for PGFPlots.

    Examples:
        AddPlot(
            color="blue",
            mark="*",
            coords=Coordinates([(0, 1), (1, 2)])
        )

        AddPlot(
            style="dashed",
            domain="0:10",
            expression="x^2"
        )
    """

    # Coordinate-based plot
    coords: Coordinates | None = None

    # Expression-based plot
    expression: str | None = None
    domain: str | None = None
    samples: int | None = None

    # Style options
    color: str | None = None
    mark: str | None = None
    style: str | None = None
    line_width: str | None = None
    only_marks: bool = False
    no_marks: bool = False
    smooth: bool = False
    thick: bool = False

    # Plot name for legend
    name: str | None = None

    # 3D options
    surf: bool = False
    mesh: bool = False

    # Error bars
    error_bars: bool = False
    error_bar_style: dict[str, Any] = field(default_factory=dict)

    # Raw options escape hatch
    _raw_options: str | None = None

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render the addplot command."""
        if scope is None:
            scope = {}

        parts = []

        # Build options
        options = {}
        if self.color:
            options["color"] = self.color
        if self.mark:
            options["mark"] = self.mark
        if self.style:
            options[self.style] = True
        if self.line_width:
            options["line width"] = self.line_width
        if self.only_marks:
            options["only marks"] = True
        if self.no_marks:
            options["mark"] = "none"
        if self.smooth:
            options["smooth"] = True
        if self.thick:
            options["thick"] = True
        if self.domain:
            options["domain"] = self.domain
        if self.samples:
            options["samples"] = self.samples
        if self.surf:
            options["surf"] = True
        if self.mesh:
            options["mesh"] = True

        # 3D variant
        plot_cmd = "\\addplot3" if self.surf or self.mesh else "\\addplot"

        # Format options string
        opts_str = format_options(options, self._raw_options)
        if opts_str:
            parts.append(f"{plot_cmd}[{opts_str}]")
        else:
            parts.append(plot_cmd)

        # Add coordinates or expression
        if self.coords is not None:
            parts.append(self.coords.render(data, scope))
        elif self.expression is not None:
            parts.append(f"{{{self.expression}}}")

        return " ".join(parts) + ";"

    def __repr__(self) -> str:
        return f"AddPlot(color={self.color!r}, mark={self.mark!r}, ...)"


@dataclass
class Legend:
    """Legend entries for a plot.

    Examples:
        Legend(["Series A", "Series B"])
        Legend([Ref("legend_label")])
    """

    entries: list[Any]

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render legend command."""
        if scope is None:
            scope = {}

        from texer.eval import _evaluate_impl

        resolved = []
        for entry in self.entries:
            resolved.append(_evaluate_impl(entry, data, scope, escape=False))

        return "\\legend{" + ", ".join(resolved) + "}"


@dataclass
class Axis:
    """A PGFPlots axis environment.

    Examples:
        Axis(
            xlabel="Time (s)",
            ylabel="Temperature (K)",
            plots=[AddPlot(...)],
            legend=["Data"]
        )
    """

    plots: list[AddPlot] = field(default_factory=list)

    # Axis labels
    xlabel: str | Spec | None = None
    ylabel: str | Spec | None = None
    zlabel: str | Spec | None = None
    title: str | Spec | None = None

    # Axis limits
    xmin: float | None = None
    xmax: float | None = None
    ymin: float | None = None
    ymax: float | None = None
    zmin: float | None = None
    zmax: float | None = None

    # Legend
    legend: list[Any] | Legend | None = None
    legend_pos: str | None = None
    legend_style: str | None = None

    # Grid
    grid: str | bool | None = None  # "major", "minor", "both", True, False

    # Axis type
    axis_type: str = "axis"  # "axis", "semilogxaxis", "semilogyaxis", "loglogaxis"

    # Scale
    width: str | None = None
    height: str | None = None

    # Other common options
    enlargelimits: bool | float | None = None
    clip: bool | None = None
    axis_lines: str | None = None  # "left", "center", "right", "box"

    # Raw options escape hatch
    _raw_options: str | None = None

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render the axis environment."""
        if scope is None:
            scope = {}

        from texer.eval import _evaluate_impl

        # Build options
        options = {}

        # Labels (resolve if Spec)
        if self.xlabel is not None:
            options["xlabel"] = _evaluate_impl(self.xlabel, data, scope, escape=False)
        if self.ylabel is not None:
            options["ylabel"] = _evaluate_impl(self.ylabel, data, scope, escape=False)
        if self.zlabel is not None:
            options["zlabel"] = _evaluate_impl(self.zlabel, data, scope, escape=False)
        if self.title is not None:
            options["title"] = _evaluate_impl(self.title, data, scope, escape=False)

        # Limits
        if self.xmin is not None:
            options["xmin"] = self.xmin
        if self.xmax is not None:
            options["xmax"] = self.xmax
        if self.ymin is not None:
            options["ymin"] = self.ymin
        if self.ymax is not None:
            options["ymax"] = self.ymax
        if self.zmin is not None:
            options["zmin"] = self.zmin
        if self.zmax is not None:
            options["zmax"] = self.zmax

        # Legend position
        if self.legend_pos is not None:
            options["legend pos"] = self.legend_pos
        if self.legend_style is not None:
            options["legend style"] = self.legend_style

        # Grid
        if self.grid is True:
            options["grid"] = "major"
        elif self.grid:
            options["grid"] = self.grid

        # Dimensions
        if self.width is not None:
            options["width"] = self.width
        if self.height is not None:
            options["height"] = self.height

        # Other options
        if self.enlargelimits is not None:
            options["enlargelimits"] = self.enlargelimits
        if self.clip is not None:
            options["clip"] = self.clip
        if self.axis_lines is not None:
            options["axis lines"] = self.axis_lines

        # Format options
        opts_str = format_options(options, self._raw_options)

        lines = []

        # Opening
        if opts_str:
            lines.append(f"\\begin{{{self.axis_type}}}[{opts_str}]")
        else:
            lines.append(f"\\begin{{{self.axis_type}}}")

        # Plots
        for plot in self.plots:
            lines.append(f"  {plot.render(data, scope)}")

        # Legend
        if self.legend is not None:
            if isinstance(self.legend, Legend):
                lines.append(f"  {self.legend.render(data, scope)}")
            else:
                legend = Legend(self.legend)
                lines.append(f"  {legend.render(data, scope)}")

        # Closing
        lines.append(f"\\end{{{self.axis_type}}}")

        return "\n".join(lines)


@dataclass
class NextGroupPlot:
    """A \\nextgroupplot command within a groupplot environment.

    Examples:
        NextGroupPlot(
            title="Plot 1",
            xlabel="X",
            plots=[AddPlot(...)]
        )
    """

    plots: list[AddPlot] = field(default_factory=list)

    # Axis labels
    xlabel: str | Spec | None = None
    ylabel: str | Spec | None = None
    zlabel: str | Spec | None = None
    title: str | Spec | None = None

    # Axis limits
    xmin: float | None = None
    xmax: float | None = None
    ymin: float | None = None
    ymax: float | None = None
    zmin: float | None = None
    zmax: float | None = None

    # Legend
    legend: list[Any] | Legend | None = None
    legend_pos: str | None = None
    legend_style: str | None = None

    # Grid
    grid: str | bool | None = None

    # Other options
    enlargelimits: bool | float | None = None
    clip: bool | None = None
    axis_lines: str | None = None

    # Raw options escape hatch
    _raw_options: str | None = None

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render the nextgroupplot command and its contents."""
        if scope is None:
            scope = {}

        from texer.eval import _evaluate_impl

        # Build options
        options = {}

        # Labels (resolve if Spec)
        if self.xlabel is not None:
            options["xlabel"] = _evaluate_impl(self.xlabel, data, scope, escape=False)
        if self.ylabel is not None:
            options["ylabel"] = _evaluate_impl(self.ylabel, data, scope, escape=False)
        if self.zlabel is not None:
            options["zlabel"] = _evaluate_impl(self.zlabel, data, scope, escape=False)
        if self.title is not None:
            options["title"] = _evaluate_impl(self.title, data, scope, escape=False)

        # Limits
        if self.xmin is not None:
            options["xmin"] = self.xmin
        if self.xmax is not None:
            options["xmax"] = self.xmax
        if self.ymin is not None:
            options["ymin"] = self.ymin
        if self.ymax is not None:
            options["ymax"] = self.ymax
        if self.zmin is not None:
            options["zmin"] = self.zmin
        if self.zmax is not None:
            options["zmax"] = self.zmax

        # Legend position
        if self.legend_pos is not None:
            options["legend pos"] = self.legend_pos
        if self.legend_style is not None:
            options["legend style"] = self.legend_style

        # Grid
        if self.grid is True:
            options["grid"] = "major"
        elif self.grid:
            options["grid"] = self.grid

        # Other options
        if self.enlargelimits is not None:
            options["enlargelimits"] = self.enlargelimits
        if self.clip is not None:
            options["clip"] = self.clip
        if self.axis_lines is not None:
            options["axis lines"] = self.axis_lines

        # Format options
        opts_str = format_options(options, self._raw_options)

        lines = []

        # Opening
        if opts_str:
            lines.append(f"\\nextgroupplot[{opts_str}]")
        else:
            lines.append("\\nextgroupplot")

        # Plots
        for plot in self.plots:
            lines.append(f"  {plot.render(data, scope)}")

        # Legend
        if self.legend is not None:
            if isinstance(self.legend, Legend):
                lines.append(f"  {self.legend.render(data, scope)}")
            else:
                legend = Legend(self.legend)
                lines.append(f"  {legend.render(data, scope)}")

        return "\n".join(lines)


@dataclass
class GroupPlot:
    """A groupplot environment for creating multiple plots in a grid layout.

    Examples:
        GroupPlot(
            group_style={"group size": "2 by 2"},
            plots=[
                NextGroupPlot(title="Plot 1", plots=[...]),
                NextGroupPlot(title="Plot 2", plots=[...]),
                NextGroupPlot(title="Plot 3", plots=[...]),
                NextGroupPlot(title="Plot 4", plots=[...]),
            ]
        )
    """

    plots: list[NextGroupPlot] = field(default_factory=list)

    # Group style options
    group_size: str | None = None  # e.g., "2 by 2"
    horizontal_sep: str | None = None
    vertical_sep: str | None = None
    xlabels_at: str | None = None  # e.g., "edge bottom"
    ylabels_at: str | None = None  # e.g., "edge left"
    xticklabels_at: str | None = None
    yticklabels_at: str | None = None

    # Common axis options (applied to all subplots)
    width: str | None = None
    height: str | None = None
    xmin: float | None = None
    xmax: float | None = None
    ymin: float | None = None
    ymax: float | None = None

    # Raw options escape hatch
    _raw_options: str | None = None
    _raw_group_style: str | None = None

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render the groupplot environment."""
        if scope is None:
            scope = {}

        # Build group style options
        group_style_opts = {}
        if self.group_size is not None:
            group_style_opts["group size"] = self.group_size
        if self.horizontal_sep is not None:
            group_style_opts["horizontal sep"] = self.horizontal_sep
        if self.vertical_sep is not None:
            group_style_opts["vertical sep"] = self.vertical_sep
        if self.xlabels_at is not None:
            group_style_opts["xlabels at"] = self.xlabels_at
        if self.ylabels_at is not None:
            group_style_opts["ylabels at"] = self.ylabels_at
        if self.xticklabels_at is not None:
            group_style_opts["xticklabels at"] = self.xticklabels_at
        if self.yticklabels_at is not None:
            group_style_opts["yticklabels at"] = self.yticklabels_at

        # Build main options
        options = {}

        # Add group style if present
        group_style_str = format_options(group_style_opts, self._raw_group_style)
        if group_style_str:
            options["group style"] = f"{{{group_style_str}}}"

        # Common options
        if self.width is not None:
            options["width"] = self.width
        if self.height is not None:
            options["height"] = self.height
        if self.xmin is not None:
            options["xmin"] = self.xmin
        if self.xmax is not None:
            options["xmax"] = self.xmax
        if self.ymin is not None:
            options["ymin"] = self.ymin
        if self.ymax is not None:
            options["ymax"] = self.ymax

        # Format options
        opts_str = format_options(options, self._raw_options)

        lines = []

        # Opening
        if opts_str:
            lines.append(f"\\begin{{groupplot}}[{opts_str}]")
        else:
            lines.append("\\begin{groupplot}")

        # Render each plot
        for plot in self.plots:
            plot_lines = plot.render(data, scope)
            for line in plot_lines.split("\n"):
                lines.append(f"  {line}" if line else line)

        # Closing
        lines.append("\\end{groupplot}")

        return "\n".join(lines)


@dataclass
class PGFPlot:
    """A complete PGFPlots tikzpicture.

    Examples:
        # Single axis
        PGFPlot(
            Axis(
                xlabel="X",
                ylabel="Y",
                plots=[AddPlot(coords=Coordinates([...]))]
            )
        )

        # Multiple plots in a grid with groupplot
        PGFPlot(
            GroupPlot(
                group_size="2 by 2",
                plots=[
                    NextGroupPlot(...),
                    NextGroupPlot(...),
                ]
            )
        )
    """

    axis: Axis | GroupPlot
    preamble: list[str] = field(default_factory=list)
    scale: float | None = None
    _raw_options: str | None = None

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render the complete tikzpicture."""
        if scope is None:
            scope = {}

        lines = []

        # Preamble (for standalone use)
        for line in self.preamble:
            lines.append(line)

        # Build tikzpicture options
        options = {}
        if self.scale is not None:
            options["scale"] = self.scale

        opts_str = format_options(options, self._raw_options)

        # Opening
        if opts_str:
            lines.append(f"\\begin{{tikzpicture}}[{opts_str}]")
        else:
            lines.append("\\begin{tikzpicture}")

        # Axis content
        axis_lines = self.axis.render(data, scope)
        for line in axis_lines.split("\n"):
            lines.append(f"  {line}" if line else line)

        # Closing
        lines.append("\\end{tikzpicture}")

        return "\n".join(lines)

    def with_preamble(self) -> str:
        """Return LaTeX code including package imports for standalone use."""
        preamble = [
            "\\documentclass{standalone}",
            "\\usepackage{pgfplots}",
            "\\pgfplotsset{compat=1.18}",
            "\\usepgfplotslibrary{groupplots}",
            "",
            "\\begin{document}",
        ]
        content = self.render({})
        closing = ["\\end{document}"]

        return "\n".join(preamble + [content] + closing)


# Convenience classes for specialized axis types
@dataclass
class SemiLogXAxis(Axis):
    """A semi-logarithmic axis (log scale on x-axis)."""

    axis_type: str = "semilogxaxis"


@dataclass
class SemiLogYAxis(Axis):
    """A semi-logarithmic axis (log scale on y-axis)."""

    axis_type: str = "semilogyaxis"


@dataclass
class LogLogAxis(Axis):
    """A log-log axis (log scale on both axes)."""

    axis_type: str = "loglogaxis"


# Helper for creating simple line plots
def simple_plot(
    x: list[float],
    y: list[float],
    xlabel: str = "x",
    ylabel: str = "y",
    title: str | None = None,
    color: str = "blue",
    mark: str = "*",
) -> PGFPlot:
    """Create a simple line plot from x and y data.

    Args:
        x: X-axis data points.
        y: Y-axis data points.
        xlabel: Label for x-axis.
        ylabel: Label for y-axis.
        title: Optional plot title.
        color: Line/marker color.
        mark: Marker style.

    Returns:
        A PGFPlot object ready for rendering.
    """
    coords = Coordinates(list(zip(x, y)))

    return PGFPlot(
        Axis(
            xlabel=xlabel,
            ylabel=ylabel,
            title=title,
            plots=[AddPlot(color=color, mark=mark, coords=coords)],
        )
    )



"""Table classes for LaTeX table generation."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any

from texer.specs import Spec, resolve_value, Iter
from texer.utils import escape_latex, format_options, indent


@dataclass
class Cell:
    """A table cell with optional formatting.

    Examples:
        Cell(Ref("value"))
        Cell(Ref("value"), bold=True)
        Cell(Format(Ref("price"), ".2f"), align="r")
        Cell(Ref("name"), bold=Cond(Ref("important"), True, False))
    """

    content: Any
    bold: bool | Spec = False
    italic: bool | Spec = False
    align: str | None = None
    _raw: bool = False

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render the cell to LaTeX."""
        from texer.eval import _evaluate_impl, evaluate_value

        content = _evaluate_impl(self.content, data, scope or {}, escape=not self._raw)

        # Resolve bold/italic if they are Specs
        bold = evaluate_value(self.bold, data, scope) if isinstance(self.bold, Spec) else self.bold
        italic = evaluate_value(self.italic, data, scope) if isinstance(self.italic, Spec) else self.italic

        if bold:
            content = f"\\textbf{{{content}}}"
        if italic:
            content = f"\\textit{{{content}}}"

        return content


@dataclass
class MultiColumn:
    """A cell spanning multiple columns.

    Examples:
        MultiColumn(3, "c", "Header Title")
    """

    ncols: int
    align: str
    content: Any

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render the multicolumn cell."""
        from texer.eval import _evaluate_impl

        content = _evaluate_impl(self.content, data, scope or {}, escape=True)
        return f"\\multicolumn{{{self.ncols}}}{{{self.align}}}{{{content}}}"


@dataclass
class MultiRow:
    """A cell spanning multiple rows.

    Examples:
        MultiRow(2, "Category")
    """

    nrows: int
    content: Any
    width: str = "*"

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render the multirow cell."""
        from texer.eval import _evaluate_impl

        content = _evaluate_impl(self.content, data, scope or {}, escape=True)
        return f"\\multirow{{{self.nrows}}}{{{self.width}}}{{{content}}}"


@dataclass
class Row:
    """A table row containing cells.

    Examples:
        Row("Name", "Value", "Unit")
        Row(Ref("name"), Ref("value"), Ref("unit"))
        Row(Cell(Ref("x"), bold=True), Ref("y"))
    """

    cells: tuple[Any, ...] = field(default_factory=tuple)
    _raw_options: str | None = None

    def __init__(self, *cells: Any, _raw_options: str | None = None):
        object.__setattr__(self, "cells", cells)
        object.__setattr__(self, "_raw_options", _raw_options)

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render the row to LaTeX."""
        from texer.eval import _evaluate_impl

        rendered_cells = []
        for cell in self.cells:
            if isinstance(cell, (Cell, MultiColumn, MultiRow)):
                rendered_cells.append(cell.render(data, scope))
            else:
                rendered_cells.append(_evaluate_impl(cell, data, scope or {}, escape=True))

        row_content = " & ".join(rendered_cells)
        return f"{row_content} \\\\"


@dataclass
class HLine:
    """A horizontal line in a table."""

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        return "\\hline"


@dataclass
class CLine:
    """A partial horizontal line (cline)."""

    start: int
    end: int

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        return f"\\cline{{{self.start}-{self.end}}}"


@dataclass
class Tabular:
    """A LaTeX tabular environment.

    Examples:
        Tabular(
            columns="lcc",
            header=Row("Name", "Value 1", "Value 2"),
            rows=Iter(Ref("data"), template=Row(Ref("name"), Ref("v1"), Ref("v2")))
        )
    """

    columns: str
    header: Row | list[Row] | None = None
    rows: Any = None  # Iter, list of Rows, or single Row
    toprule: bool = False
    midrule: bool = False
    bottomrule: bool = False
    _raw_options: str | None = None

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render the tabular environment."""
        if scope is None:
            scope = {}

        lines = []

        # Opening
        lines.append(f"\\begin{{tabular}}{{{self.columns}}}")

        # Top rule (booktabs)
        if self.toprule:
            lines.append("  \\toprule")

        # Header
        if self.header is not None:
            if isinstance(self.header, list):
                for h in self.header:
                    lines.append(f"  {h.render(data, scope)}")
            else:
                lines.append(f"  {self.header.render(data, scope)}")

            # Mid rule after header
            if self.midrule or self.toprule:
                lines.append("  \\midrule")

        # Body rows
        if self.rows is not None:
            rendered_rows = self._render_rows(data, scope)
            for row in rendered_rows:
                lines.append(f"  {row}")

        # Bottom rule
        if self.bottomrule:
            lines.append("  \\bottomrule")

        # Closing
        lines.append("\\end{tabular}")

        return "\n".join(lines)

    def _render_rows(self, data: Any, scope: dict[str, Any]) -> list[str]:
        """Render the body rows."""
        if isinstance(self.rows, Iter):
            # For Iter with a Row template, we need to handle specially
            # Get the source items
            if isinstance(self.rows.source, str):
                import glom
                items = glom.glom(data, self.rows.source)
            else:
                items = self.rows.source.resolve(data, scope)

            if self.rows.template is not None:
                # Render the template for each item
                results = []
                for item in items:
                    if isinstance(self.rows.template, Row):
                        results.append(self.rows.template.render(item, scope))
                    elif hasattr(self.rows.template, "render"):
                        results.append(self.rows.template.render(item, scope))
                    else:
                        from texer.eval import _evaluate_impl
                        results.append(_evaluate_impl(self.rows.template, item, scope, escape=True))
                return results
            else:
                # No template, just resolve
                return [str(item) for item in items]
        elif isinstance(self.rows, list):
            return [row.render(data, scope) for row in self.rows]
        elif isinstance(self.rows, Row):
            return [self.rows.render(data, scope)]
        elif isinstance(self.rows, Spec):
            resolved = self.rows.resolve(data, scope)
            if isinstance(resolved, list):
                return [str(r) for r in resolved]
            return [str(resolved)]
        return []


@dataclass
class Table:
    """A LaTeX table environment (floating).

    Examples:
        Table(
            Tabular(...),
            caption="My Table",
            label="tab:mytable",
            position="htbp"
        )
    """

    content: Tabular
    caption: Any = None
    label: str | None = None
    position: str = "htbp"
    centering: bool = True
    _raw_options: str | None = None

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render the table environment."""
        if scope is None:
            scope = {}

        lines = []

        # Opening
        lines.append(f"\\begin{{table}}[{self.position}]")

        if self.centering:
            lines.append("  \\centering")

        # Caption (before table for some styles)
        if self.caption is not None:
            from texer.eval import _evaluate_impl
            caption_text = _evaluate_impl(self.caption, data, scope, escape=True)
            lines.append(f"  \\caption{{{caption_text}}}")

        # Label
        if self.label is not None:
            lines.append(f"  \\label{{{self.label}}}")

        # Tabular content
        tabular_lines = self.content.render(data, scope)
        for line in tabular_lines.split("\n"):
            lines.append(f"  {line}" if line else line)

        # Closing
        lines.append("\\end{table}")

        return "\n".join(lines)


# Convenience function for simple tables
def simple_table(
    headers: list[str],
    rows: list[list[Any]],
    caption: str | None = None,
    label: str | None = None,
) -> Table:
    """Create a simple table from headers and row data.

    Args:
        headers: List of column headers.
        rows: List of row data (each row is a list of values).
        caption: Optional table caption.
        label: Optional label for referencing.

    Returns:
        A Table object ready for rendering.
    """
    ncols = len(headers)
    columns = "l" + "c" * (ncols - 1)  # First column left, rest centered

    header_row = Row(*headers)
    data_rows = [Row(*row) for row in rows]

    tabular = Tabular(
        columns=columns,
        header=header_row,
        rows=data_rows,
        toprule=True,
        midrule=True,
        bottomrule=True,
    )

    return Table(tabular, caption=caption, label=label)



"""Core spec system for texer - glom-style specs with type checking."""

from __future__ import annotations

from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from typing import Any, Callable, Generic, TypeVar, Union

import glom

T = TypeVar("T")
U = TypeVar("U")


class Spec(ABC):
    """Base class for all specs. Specs are lazy evaluation descriptors."""

    @abstractmethod
    def resolve(self, data: Any, scope: dict[str, Any] | None = None) -> Any:
        """Resolve this spec against the given data."""
        ...

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(...)"


@dataclass(frozen=True)
class Ref(Spec):
    """Reference to a data path using glom-style dot notation.

    Examples:
        Ref("name")           -> data["name"]
        Ref("user.email")     -> data["user"]["email"]
        Ref("items.0.value")  -> data["items"][0]["value"]
    """

    path: str
    default: Any = field(default=None)

    def resolve(self, data: Any, scope: dict[str, Any] | None = None) -> Any:
        """Resolve the reference path against data."""
        # Check scope first for locally bound variables
        if scope and self.path in scope:
            return scope[self.path]

        # Use glom for path resolution
        try:
            return glom.glom(data, self.path)
        except glom.PathAccessError:
            if self.default is not None:
                return self.default
            raise

    def __repr__(self) -> str:
        if self.default is not None:
            return f'Ref("{self.path}", default={self.default!r})'
        return f'Ref("{self.path}")'

    # Comparison operators for Cond
    def __gt__(self, other: Any) -> Comparison:
        return Comparison(self, ">", other)

    def __lt__(self, other: Any) -> Comparison:
        return Comparison(self, "<", other)

    def __ge__(self, other: Any) -> Comparison:
        return Comparison(self, ">=", other)

    def __le__(self, other: Any) -> Comparison:
        return Comparison(self, "<=", other)

    def __eq__(self, other: Any) -> Comparison:  # type: ignore[override]
        return Comparison(self, "==", other)

    def __ne__(self, other: Any) -> Comparison:  # type: ignore[override]
        return Comparison(self, "!=", other)


@dataclass(frozen=True)
class Comparison(Spec):
    """A comparison expression for use in Cond."""

    left: Spec
    op: str
    right: Any

    def resolve(self, data: Any, scope: dict[str, Any] | None = None) -> bool:
        """Evaluate the comparison."""
        left_val = resolve_value(self.left, data, scope)
        right_val = resolve_value(self.right, data, scope)

        ops = {
            ">": lambda a, b: a > b,
            "<": lambda a, b: a < b,
            ">=": lambda a, b: a >= b,
            "<=": lambda a, b: a <= b,
            "==": lambda a, b: a == b,
            "!=": lambda a, b: a != b,
        }
        return ops[self.op](left_val, right_val)

    def __repr__(self) -> str:
        return f"({self.left!r} {self.op} {self.right!r})"

    def __and__(self, other: Comparison) -> And:
        return And(self, other)

    def __or__(self, other: Comparison) -> Or:
        return Or(self, other)


@dataclass(frozen=True)
class And(Spec):
    """Logical AND of two conditions."""

    left: Spec
    right: Spec

    def resolve(self, data: Any, scope: dict[str, Any] | None = None) -> bool:
        return bool(resolve_value(self.left, data, scope)) and bool(
            resolve_value(self.right, data, scope)
        )


@dataclass(frozen=True)
class Or(Spec):
    """Logical OR of two conditions."""

    left: Spec
    right: Spec

    def resolve(self, data: Any, scope: dict[str, Any] | None = None) -> bool:
        return bool(resolve_value(self.left, data, scope)) or bool(
            resolve_value(self.right, data, scope)
        )


@dataclass(frozen=True)
class Iter(Spec):
    """Iterate over a collection, applying a template to each item.

    Examples:
        Iter(Ref("items"), template=Row(Ref("name"), Ref("value")))
        Iter(Ref("points"), x=Ref("x"), y=Ref("y"))  # For coordinates
    """

    source: Spec | str
    template: Any = None
    separator: str = "\n"
    # For coordinate-style iteration
    x: Spec | None = None
    y: Spec | None = None
    z: Spec | None = None

    def resolve(self, data: Any, scope: dict[str, Any] | None = None) -> Any:
        """Resolve by iterating over source and applying template."""
        # Resolve the source collection
        if isinstance(self.source, str):
            items = glom.glom(data, self.source)
        else:
            items = self.source.resolve(data, scope)

        if not hasattr(items, "__iter__"):
            raise TypeError(f"Iter source must be iterable, got {type(items)}")

        results = []
        for item in items:
            # Create a new scope with the current item as the data context
            item_scope = dict(scope) if scope else {}

            if self.template is not None:
                # Template mode: resolve template against each item
                result = resolve_value(self.template, item, item_scope)
                results.append(result)
            elif self.x is not None:
                # Coordinate mode: extract x, y, z from each item
                x_val = resolve_value(self.x, item, item_scope)
                y_val = resolve_value(self.y, item, item_scope) if self.y else None
                z_val = resolve_value(self.z, item, item_scope) if self.z else None

                if z_val is not None:
                    results.append((x_val, y_val, z_val))
                elif y_val is not None:
                    results.append((x_val, y_val))
                else:
                    results.append(x_val)

        return results

    def __repr__(self) -> str:
        parts = [f"source={self.source!r}"]
        if self.template is not None:
            parts.append(f"template={self.template!r}")
        if self.x is not None:
            parts.append(f"x={self.x!r}")
        if self.y is not None:
            parts.append(f"y={self.y!r}")
        if self.z is not None:
            parts.append(f"z={self.z!r}")
        return f"Iter({', '.join(parts)})"


@dataclass(frozen=True)
class Format(Spec):
    """Format a value using Python format specification.

    Examples:
        Format(Ref("value"), ".2f")     -> f"{value:.2f}"
        Format(Ref("pct"), ".1%")       -> f"{pct:.1%}"
        Format(Ref("num"), "04d")       -> f"{num:04d}"
    """

    value: Spec | Any
    fmt: str

    def resolve(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Resolve and format the value."""
        val = resolve_value(self.value, data, scope)
        return format(val, self.fmt)

    def __repr__(self) -> str:
        return f'Format({self.value!r}, "{self.fmt}")'


@dataclass(frozen=True)
class Cond(Spec):
    """Conditional logic - returns one value or another based on condition.

    Examples:
        Cond(Ref("x") > 5, "red", "blue")
        Cond(Ref("active"), "\\checkmark", "")
    """

    condition: Spec | bool
    if_true: Any
    if_false: Any = ""

    def resolve(self, data: Any, scope: dict[str, Any] | None = None) -> Any:
        """Evaluate condition and return the selected branch (not resolved).

        Note: This returns the branch itself, not its resolved value.
        This allows Raw and other special specs to be handled properly
        by the evaluation layer.
        """
        cond_result = resolve_value(self.condition, data, scope)
        if cond_result:
            return self.if_true
        return self.if_false

    def __repr__(self) -> str:
        return f"Cond({self.condition!r}, {self.if_true!r}, {self.if_false!r})"


@dataclass(frozen=True)
class Literal(Spec):
    """A literal value that doesn't need resolution.

    Examples:
        Literal("some text")
        Literal(42)
    """

    value: Any

    def resolve(self, data: Any, scope: dict[str, Any] | None = None) -> Any:
        return self.value

    def __repr__(self) -> str:
        return f"Literal({self.value!r})"


@dataclass(frozen=True)
class Raw(Spec):
    r"""Raw LaTeX code that should not be escaped.

    Examples:
        Raw(r"\textbf{bold}")
        Raw(r"\hline")
    """

    latex: str

    def resolve(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        return self.latex

    def __repr__(self) -> str:
        return f"Raw({self.latex!r})"

    @property
    def is_raw(self) -> bool:
        return True


@dataclass(frozen=True)
class Call(Spec):
    """Call a function with resolved arguments.

    Examples:
        Call(len, Ref("items"))
        Call(max, Ref("values"))
    """

    func: Callable[..., Any]
    args: tuple[Any, ...] = field(default_factory=tuple)
    kwargs: dict[str, Any] = field(default_factory=dict)

    def resolve(self, data: Any, scope: dict[str, Any] | None = None) -> Any:
        resolved_args = [resolve_value(arg, data, scope) for arg in self.args]
        resolved_kwargs = {
            k: resolve_value(v, data, scope) for k, v in self.kwargs.items()
        }
        return self.func(*resolved_args, **resolved_kwargs)

    def __repr__(self) -> str:
        return f"Call({self.func.__name__}, {self.args!r}, {self.kwargs!r})"


@dataclass(frozen=True)
class Join(Spec):
    """Join multiple specs with a separator.

    Examples:
        Join([Ref("first"), Ref("last")], " ")
    """

    parts: list[Any]
    separator: str = ""

    def resolve(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        resolved = [str(resolve_value(p, data, scope)) for p in self.parts]
        return self.separator.join(resolved)


def resolve_value(value: Any, data: Any, scope: dict[str, Any] | None = None) -> Any:
    """Resolve a value which may be a Spec or a plain value."""
    if isinstance(value, Spec):
        return value.resolve(data, scope)
    return value



"""Evaluation engine for texer specs and LaTeX elements."""

from __future__ import annotations

from typing import Any, Protocol, runtime_checkable

from texer.specs import Spec, resolve_value, Raw
from texer.utils import escape_latex


@runtime_checkable
class Renderable(Protocol):
    """Protocol for objects that can render to LaTeX."""

    def render(self, data: Any, scope: dict[str, Any] | None = None) -> str:
        """Render this object to LaTeX string."""
        ...


def evaluate(
    element: Any,
    data: Any | None = None,
    scope: dict[str, Any] | None = None,
    escape: bool = True,
) -> str:
    """Evaluate an element and return LaTeX string.

    This is the main entry point for converting texer elements to LaTeX.

    Args:
        element: The element to evaluate (Spec, Renderable, or plain value).
        data: The data context for resolving specs.
        scope: Additional scope variables.
        escape: Whether to escape LaTeX special characters in strings.

    Returns:
        LaTeX string representation.

    Examples:
        >>> from texer import Ref, Table, Tabular, Row
        >>> data = {"name": "Alice", "value": 42}
        >>> evaluate(Ref("name"), data)
        'Alice'
    """
    if data is None:
        data = {}

    if scope is None:
        scope = {}

    return _evaluate_impl(element, data, scope, escape)


def _evaluate_impl(
    element: Any,
    data: Any,
    scope: dict[str, Any],
    escape: bool,
) -> str:
    """Internal implementation of evaluate."""
    # Handle None
    if element is None:
        return ""

    # Handle Raw specs (don't escape)
    if isinstance(element, Raw):
        return element.resolve(data, scope)

    # Handle Specs
    if isinstance(element, Spec):
        resolved = element.resolve(data, scope)
        return _evaluate_impl(resolved, data, scope, escape)

    # Handle Renderables (Table, Tabular, Row, etc.)
    if isinstance(element, Renderable):
        return element.render(data, scope)

    # Handle lists/tuples
    if isinstance(element, (list, tuple)):
        return "".join(_evaluate_impl(item, data, scope, escape) for item in element)

    # Handle plain values
    result = str(element)
    if escape:
        return escape_latex(result)
    return result


def evaluate_value(
    value: Any,
    data: Any,
    scope: dict[str, Any] | None = None,
) -> Any:
    """Evaluate a value without converting to string.

    This resolves Specs but doesn't convert to LaTeX string.
    Useful for getting raw values (lists, numbers, etc.)

    Args:
        value: The value to evaluate.
        data: The data context.
        scope: Additional scope variables.

    Returns:
        The resolved value (may be any type).
    """
    if scope is None:
        scope = {}

    return resolve_value(value, data, scope)



"""Utility functions for LaTeX generation."""

from __future__ import annotations

import re
from typing import Any

# Characters that need escaping in LaTeX
LATEX_SPECIAL_CHARS = {
    "&": r"\&",
    "%": r"\%",
    "$": r"\$",
    "#": r"\#",
    "_": r"\_",
    "{": r"\{",
    "}": r"\}",
    "~": r"\textasciitilde{}",
    "^": r"\textasciicircum{}",
    "\\": r"\textbackslash{}",
}

# Regex pattern for special characters
LATEX_ESCAPE_PATTERN = re.compile(
    "|".join(re.escape(char) for char in LATEX_SPECIAL_CHARS.keys())
)


def escape_latex(text: str) -> str:
    """Escape special LaTeX characters in text.

    Args:
        text: The text to escape.

    Returns:
        The escaped text safe for LaTeX.

    Examples:
        >>> escape_latex("10% off")
        '10\\% off'
        >>> escape_latex("$100")
        '\\$100'
    """
    return LATEX_ESCAPE_PATTERN.sub(
        lambda m: LATEX_SPECIAL_CHARS[m.group()], text
    )


def format_option_value(value: Any) -> str:
    """Format a Python value for use in LaTeX options.

    Args:
        value: The value to format.

    Returns:
        LaTeX-formatted string.

    Examples:
        >>> format_option_value("north west")
        '{north west}'
        >>> format_option_value(True)
        'true'
        >>> format_option_value(3.14)
        '3.14'
    """
    if value is True:
        return "true"
    if value is False:
        return "false"
    if value is None:
        return ""
    if isinstance(value, (int, float)):
        return str(value)
    # Strings get wrapped in braces if they contain spaces or special chars
    s = str(value)
    if " " in s or any(c in s for c in ",=[]"):
        return f"{{{s}}}"
    return s


def format_options(options: dict[str, Any], raw_options: str | None = None) -> str:
    """Format a dictionary of options for LaTeX.

    Args:
        options: Dictionary of option key-value pairs.
        raw_options: Raw LaTeX options string to append.

    Returns:
        Formatted options string (without surrounding brackets).

    Examples:
        >>> format_options({"xlabel": "Time", "ylabel": "Value"})
        'xlabel={Time}, ylabel={Value}'
    """
    parts = []
    for key, value in options.items():
        if value is None:
            continue
        if value is True:
            # Boolean true options are just the key
            parts.append(key)
        elif value is False:
            continue  # Skip false options
        else:
            formatted = format_option_value(value)
            # Convert Python-style names to LaTeX style (legend_pos -> legend pos)
            latex_key = key.replace("_", " ")
            parts.append(f"{latex_key}={formatted}")

    if raw_options:
        parts.append(raw_options)

    return ", ".join(parts)


def indent(text: str, spaces: int = 2) -> str:
    """Indent each line of text.

    Args:
        text: The text to indent.
        spaces: Number of spaces to indent.

    Returns:
        Indented text.
    """
    prefix = " " * spaces
    return "\n".join(prefix + line if line else line for line in text.split("\n"))


def wrap_environment(name: str, content: str, options: str = "") -> str:
    """Wrap content in a LaTeX environment.

    Args:
        name: Environment name.
        content: Content to wrap.
        options: Optional options string.

    Returns:
        Complete environment string.
    """
    if options:
        begin = f"\\begin{{{name}}}[{options}]"
    else:
        begin = f"\\begin{{{name}}}"
    end = f"\\end{{{name}}}"
    return f"{begin}\n{indent(content)}\n{end}"



"""Tests for table generation."""

import pytest
from texer import Table, Tabular, Row, Cell, MultiColumn, Ref, Iter, evaluate


class TestRow:
    def test_simple_row(self):
        row = Row("A", "B", "C")
        result = row.render({})
        assert result == "A & B & C \\\\"

    def test_row_with_ref(self):
        row = Row(Ref("name"), Ref("value"))
        data = {"name": "Test", "value": 42}
        result = row.render(data)
        assert result == "Test & 42 \\\\"


class TestCell:
    def test_bold_cell(self):
        cell = Cell("text", bold=True)
        result = cell.render({})
        assert result == "\\textbf{text}"

    def test_italic_cell(self):
        cell = Cell("text", italic=True)
        result = cell.render({})
        assert result == "\\textit{text}"

    def test_bold_italic_cell(self):
        cell = Cell("text", bold=True, italic=True)
        result = cell.render({})
        assert result == "\\textit{\\textbf{text}}"


class TestMultiColumn:
    def test_multicolumn(self):
        mc = MultiColumn(3, "c", "Header")
        result = mc.render({})
        assert result == "\\multicolumn{3}{c}{Header}"


class TestTabular:
    def test_simple_tabular(self):
        tabular = Tabular(
            columns="lcc",
            header=Row("Name", "Value 1", "Value 2"),
            rows=[Row("Item", "10", "20")],
        )
        result = tabular.render({})
        assert "\\begin{tabular}{lcc}" in result
        assert "Name & Value 1 & Value 2 \\\\" in result
        assert "Item & 10 & 20 \\\\" in result
        assert "\\end{tabular}" in result

    def test_tabular_with_booktabs(self):
        tabular = Tabular(
            columns="lc",
            header=Row("A", "B"),
            rows=[Row("1", "2")],
            toprule=True,
            bottomrule=True,
        )
        result = tabular.render({})
        assert "\\toprule" in result
        assert "\\midrule" in result
        assert "\\bottomrule" in result

    def test_tabular_with_iter(self):
        tabular = Tabular(
            columns="lc",
            header=Row("Name", "Value"),
            rows=Iter(Ref("data"), template=Row(Ref("name"), Ref("value"))),
        )
        data = {
            "data": [
                {"name": "Item 1", "value": 10},
                {"name": "Item 2", "value": 20},
            ]
        }
        result = tabular.render(data)
        assert "Item 1 & 10 \\\\" in result
        assert "Item 2 & 20 \\\\" in result


class TestTable:
    def test_table_environment(self):
        table = Table(
            Tabular(columns="lc", rows=[Row("A", "B")]),
            caption="My Table",
            label="tab:test",
        )
        result = table.render({})
        assert "\\begin{table}[htbp]" in result
        assert "\\centering" in result
        assert "\\caption{My Table}" in result
        assert "\\label{tab:test}" in result
        assert "\\end{table}" in result


class TestEvaluate:
    def test_evaluate_table(self):
        table = Table(
            Tabular(
                columns="lcc",
                header=Row("Name", "V1", "V2"),
                rows=Iter(
                    Ref("items"),
                    template=Row(Ref("name"), Ref("v1"), Ref("v2")),
                ),
                toprule=True,
                bottomrule=True,
            ),
            caption=Ref("title"),
            label="tab:data",
        )
        data = {
            "title": "Results",
            "items": [
                {"name": "A", "v1": 1, "v2": 2},
                {"name": "B", "v1": 3, "v2": 4},
            ],
        }
        result = evaluate(table, data)
        assert "\\caption{Results}" in result
        assert "A & 1 & 2 \\\\" in result
        assert "B & 3 & 4 \\\\" in result
